function [out,b] = FastSegmentation(in, varargin)
% Parameters 
ip = inputParser;
addParameter(ip,'constrained', 1);
addParameter(ip,'alpha_ratio', 0.5);
addParameter(ip,'mu', 5e-3);
addParameter(ip,'up_factor', 2);
addParameter(ip,'blur_std', 1.0);
addParameter(ip,'noise_level', 0.01);
parse(ip, varargin{:});
par = ip.Results;

constrained = convert_param(par.constrained);
alpha_ratio = convert_param(par.alpha_ratio);
mu          = convert_param(par.mu);
up_factor   = convert_param(par.up_factor);
blur_std    = convert_param(par.blur_std);
noise_level = convert_param(par.noise_level);

% Real Image
gt=im2double(in);
gt=modcrop(gt,[up_factor,up_factor]);
HRsize = size(gt);

% Point-Spread-Function and Blur image
if blur_std > 0
    kernel = fspecial('gaussian', [9 9], blur_std);
else
    kernel = 1.0;
end

PSFfft = psf2otf(kernel, HRsize);
gt_fft = fft2(gt);
b = real(ifft2(PSFfft .* gt_fft));

% Downsampled image and Noisy image
b   = b(1:up_factor:end, 1:up_factor:end, :);
eta = randn(size(b));
b   = b + noise_level * eta;

% Algorithm parameters
mu_t       = mu;
rel_chg_th = 5e-4;
itr_th     = 200;
alpha      = fix((HRsize(1) * HRsize(2)) * alpha_ratio);
alpha_th   = alpha + alpha * 0.05;
debug = 0;

% Starting Point
x = imresize(b,up_factor,'lanczos3');

if constrained
     [res] = GradL0_C_ADMM_FSR(b, kernel,up_factor,x,alpha,alpha_th,itr_th,debug,gt);
     out = res.x;
     fprintf("GradL0_Constrained_ADMM_FSR: %d\n", res.l0(end));
else
     [res] = GradL0_U_ADMM_FSR(b, kernel,up_factor,x,mu_t,rel_chg_th,itr_th,debug,gt);
     out = res.x;
     fprintf("GradL0_Unconstrained_ADMM_FSR: %d\n", res.l0(end));
end

end

function [out]=convert_param(in)
    if ~isa(in,'numeric')
        out = str2num(in);
    else
        out = in;
    end
end